@@ -1,6 +1,6 @@
-/*  xfce4
+/*  $Id: groups.c 4135 2004-09-13 20:31:39Z jasper $
  *  
- *  Copyright (C) 2002 Jasper Huijsmans (huysmans@users.sourceforge.net)
+ *  Copyright (C) 2002-2004 Jasper Huijsmans (jasper@xfce.org)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,31 +17,36 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-/*  Groups
- *  ------
- *  The panel consist of a collection of panel groups. A panel group defines 
- *  several containers to pack widgets into and to allow easy rearragement of 
- *  these widgets.
- *  
- *  Two widgets are packed into a panel group: a panel control and a popup 
- *  button. 
-*/
+/**
+ * Groups
+ * ------
+ * The panel consist of a collection of panel groups. 
+ *
+ * A PanelGroup contains two items: a panel control and a popup button. 
+ *
+ * groups_* functions are for general houskeeping and to perform actions on
+ * all panel groups.
+ *
+ * panel_group_* funtions act on a single PanelGroup and handle things like
+ * creation destruction and layout.
+ **/
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <libxfce4util/i18n.h>
-#include <libxfce4util/debug.h>
+#include <gmodule.h>
+#include <libxfce4util/libxfce4util.h>
 
 #include "xfce.h"
 #include "groups.h"
 #include "popup.h"
 
-static GSList *group_list = NULL;
-static GSList *removed_list = NULL;
-static GtkArrowType popup_arrow_type = GTK_ARROW_UP;
+/* defined in controls.c */
+extern void control_class_unref (ControlClass * cclass);
 
+static GSList *group_list = NULL;
+static GtkArrowType popup_arrow_type = GTK_ARROW_UP;
 static GtkBox *groupbox;
 
 /*  Panel group
@@ -54,7 +59,7 @@ struct _PanelGroup
     int index;
 
     GtkWidget *base;		/* container to pack into panel */
-    GtkWidget *box;		/* hbox or vbox */
+    GtkWidget *box;
 
     PanelPopup *popup;
     Control *control;
@@ -67,7 +72,7 @@ panel_group_arrange (PanelGroup * pg)
 
     if (pg->box)
     {
-	if (pg->popup)
+	if (pg->popup && pg->popup->button->parent)
 	    panel_popup_unpack (pg->popup);
 	if (pg->control)
 	    control_unpack (pg->control);
@@ -99,24 +104,21 @@ panel_group_arrange (PanelGroup * pg)
     }
 }
 
+G_MODULE_EXPORT /* EXPORT:create_panel_group */
 PanelGroup *
 create_panel_group (int index)
 {
-    PanelGroup *pg = g_new (PanelGroup, 1);
+    PanelGroup *pg = g_new0 (PanelGroup, 1);
 
     pg->index = index;
 
-    pg->base = gtk_alignment_new (0, 0, 1, 1);
+    pg->base = gtk_alignment_new (0.5, 0.5, 1, 1);
     gtk_widget_show (pg->base);
 
     /* protect against destruction when unpacking */
     g_object_ref (pg->base);
     gtk_object_sink (GTK_OBJECT (pg->base));
 
-    pg->box = NULL;
-    pg->popup = NULL;
-    pg->control = NULL;
-
     return pg;
 }
 
@@ -142,10 +144,13 @@ panel_group_unpack (PanelGroup * pg)
     gtk_container_remove (GTK_CONTAINER (pg->base->parent), pg->base);
 }
 
+
 /*  Groups
  *  ------
  *  Mainly housekeeping for the global list of panel groups
 */
+
+G_MODULE_EXPORT /* EXPORT:groups_init */
 void
 groups_init (GtkBox * box)
 {
@@ -155,6 +160,7 @@ groups_init (GtkBox * box)
     control_class_list_init ();
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_cleanup */
 void
 groups_cleanup (void)
 {
@@ -171,11 +177,45 @@ groups_cleanup (void)
     control_class_list_cleanup ();
 
     g_slist_free (group_list);
-    g_slist_free (removed_list);
     group_list = NULL;
-    removed_list = NULL;
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_pack */
+void
+groups_pack (GtkBox * box)
+{
+    GSList *li;
+    PanelGroup *group;
+
+    groupbox = box;
+
+    for (li = group_list; li; li = li->next)
+    {
+	group = li->data;
+
+	panel_group_pack (group, box);
+	panel_group_arrange (group);
+    }
+}
+
+G_MODULE_EXPORT /* EXPORT:groups_unpack */
+void
+groups_unpack (void)
+{
+    GSList *li;
+    PanelGroup *group;
+
+    for (li = group_list; li; li = li->next)
+    {
+	group = li->data;
+
+	panel_group_unpack (group);
+    }
+}
+
+/* configuration */
+
+G_MODULE_EXPORT /* EXPORT:old_groups_set_from_xml */
 void
 old_groups_set_from_xml (int side, xmlNodePtr node)
 {
@@ -194,7 +234,7 @@ old_groups_set_from_xml (int side, xmlNodePtr node)
     if (node)
 	node = node->children;
 
-    for (i = last_group; /*i < settings.num_groups ||*/ li || node; i++)
+    for (i = last_group; /*i < settings.num_groups || */ li || node; i++)
     {
 	gboolean control_created = FALSE;
 
@@ -212,7 +252,6 @@ old_groups_set_from_xml (int side, xmlNodePtr node)
 	    group_list = g_slist_append (group_list, group);
 
 	    group->popup = create_panel_popup ();
-
 	    group->control = control_new (i);
 
 	    panel_group_arrange (group);
@@ -249,76 +288,65 @@ old_groups_set_from_xml (int side, xmlNodePtr node)
     }
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_set_from_xml */
 void
 groups_set_from_xml (xmlNodePtr node)
 {
-    xmlNodePtr child;
     int i;
-    GSList *li;
-    PanelGroup *group;
-
-    li = group_list;
 
     /* children are "Group" nodes */
     if (node)
 	node = node->children;
 
-    DBG("expected number of panel items: %d", settings.num_groups);
-    
-    for (i = 0; /* i < settings.num_groups || */ li || node; i++)
+    for (i = 0; node; i++, node = node->next)
     {
 	gboolean control_created = FALSE;
+	xmlNodePtr child, popup_node = NULL;
+	PanelGroup *group;
 
-	if (li)
+	group = create_panel_group (i);
+	panel_group_pack (group, groupbox);
+	group_list = g_slist_append (group_list, group);
+
+	group->control = control_new (i);
+
+	panel_group_arrange (group);
+
+	for (child = node->children; child; child = child->next)
 	{
-	    group = li->data;
-	}
-	else
-	{
-	    group = create_panel_group (i);
-	    panel_group_pack (group, groupbox);
-	    group_list = g_slist_append (group_list, group);
-
-	    group->popup = create_panel_popup ();
-
-	    group->control = control_new (i);
-
-	    panel_group_arrange (group);
-	}
-
-	if (node)
-	{
-	    for (child = node->children; child; child = child->next)
+	    /* create popup items and panel control */
+	    if (xmlStrEqual (child->name, (const xmlChar *) "Popup"))
 	    {
-		/* create popup items and panel control */
-		if (xmlStrEqual (child->name, (const xmlChar *) "Popup"))
+		popup_node = child;
+	    }
+	    else if (xmlStrEqual (child->name, (const xmlChar *) "Control"))
+	    {
+		/* TODO: make this part of control creation */
+		if (popup_node)
 		{
-		    panel_popup_set_from_xml (group->popup, child);
+		    group->popup = create_panel_popup ();
+		    panel_group_arrange (group);
+		    gtk_widget_hide (group->popup->button);
+		    panel_popup_set_from_xml (group->popup, popup_node);
 		}
-		else if (xmlStrEqual
-			 (child->name, (const xmlChar *) "Control"))
-		{
+
+		control_created =
 		    control_set_from_xml (group->control, child);
-		    control_created = TRUE;
-		}
+
+		break;
 	    }
 	}
 
 	if (!control_created)
-	    control_set_from_xml (group->control, NULL);
-
-	if (node)
-	    node = node->next;
-
-	if (li)
-	    li = li->next;
+	{
+	    group_list = g_slist_remove (group_list, group);
+	    panel_group_unpack (group);
+	    panel_group_free (group);
+	}
     }
-
-    settings.num_groups = g_slist_length (group_list);
-    
-    DBG("actual number of panel items: %d", settings.num_groups);
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_write_xml */
 void
 groups_write_xml (xmlNodePtr root)
 {
@@ -334,30 +362,79 @@ groups_write_xml (xmlNodePtr root)
 
 	child = xmlNewTextChild (node, NULL, "Group", NULL);
 
-	panel_popup_write_xml (group->popup, child);
+	if (group->control->with_popup && group->popup)
+	    panel_popup_write_xml (group->popup, child);
+
 	control_write_xml (group->control, child);
     }
 }
 
+/* settings */
+G_MODULE_EXPORT /* EXPORT:groups_set_orientation */
 void
-groups_pack (GtkBox * box)
+groups_set_orientation (int orientation)
+{
+    GSList *li;
+    PanelGroup *group;
+
+    for (li = group_list; li; li = li->next)
+    {
+	group = li->data;
+
+	control_set_orientation (group->control, orientation);
+    }
+}
+
+G_MODULE_EXPORT /* EXPORT:groups_set_layer */
+void
+groups_set_layer (int layer)
+{
+    GSList *li;
+    PanelGroup *group;
+
+    for (li = group_list; li; li = li->next)
+    {
+	group = li->data;
+
+	panel_popup_set_layer (group->popup, layer);
+    }
+}
+
+G_MODULE_EXPORT /* EXPORT:groups_set_size */
+void
+groups_set_size (int size)
+{
+    GSList *li;
+    PanelGroup *group;
+
+    for (li = group_list; li; li = li->next)
+    {
+	group = li->data;
+
+	panel_popup_set_size (group->popup, size);
+	control_set_size (group->control, size);
+    }
+}
+
+G_MODULE_EXPORT /* EXPORT:groups_set_popup_position */
+void
+groups_set_popup_position (int position)
 {
     GSList *li;
     PanelGroup *group;
 
-    groupbox = box;
-
     for (li = group_list; li; li = li->next)
     {
 	group = li->data;
 
-	panel_group_pack (group, box);
 	panel_group_arrange (group);
+	panel_popup_set_popup_position (group->popup, position);
     }
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_set_theme */
 void
-groups_unpack (void)
+groups_set_theme (const char *theme)
 {
     GSList *li;
     PanelGroup *group;
@@ -366,27 +443,39 @@ groups_unpack (void)
     {
 	group = li->data;
 
-	panel_group_unpack (group);
+	panel_popup_set_theme (group->popup, theme);
+	control_set_theme (group->control, theme);
     }
 }
 
+/* arrow direction */
+
+G_MODULE_EXPORT /* EXPORT:groups_set_arrow_direction */
 void
-groups_register_control (Control * control)
+groups_set_arrow_direction (GtkArrowType type)
 {
+    GSList *li;
     PanelGroup *group;
 
-    group = g_slist_nth (group_list, control->index)->data;
+    popup_arrow_type = type;
 
-    group->control = control;
+    for (li = group_list; li; li = li->next)
+    {
+	group = li->data;
 
-    if (!control->with_popup)
-	groups_show_popup (control->index, FALSE);
-    else
-	groups_show_popup (control->index, TRUE);
-
-    panel_group_arrange (group);
+	panel_popup_set_arrow_type (group->popup, type);
+    }
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_get_arrow_direction */
+GtkArrowType
+groups_get_arrow_direction (void)
+{
+    return popup_arrow_type;
+}
+
+/* find or act on specific groups */
+
 static PanelGroup *
 groups_get_nth (int n)
 {
@@ -410,6 +499,7 @@ groups_get_nth (int n)
     return group;
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_get_control */
 Control *
 groups_get_control (int index)
 {
@@ -419,6 +509,7 @@ groups_get_control (int index)
     return group->control;
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_get_popup */
 PanelPopup *
 groups_get_popup (int index)
 {
@@ -428,6 +519,7 @@ groups_get_popup (int index)
     return group->popup;
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_move */
 void
 groups_move (int from, int to)
 {
@@ -454,6 +546,7 @@ groups_move (int from, int to)
     }
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_remove */
 void
 groups_remove (int index)
 {
@@ -463,11 +556,14 @@ groups_remove (int index)
 
     li = g_slist_nth (group_list, index);
     /* Paranoid, should not happen here */
-    if (!li) 
-        return;
- 
+    if (!li)
+	return;
+
     group = li->data;
 
+    DBG ("unref class %s", group->control->cclass->caption);
+    control_class_unref (group->control->cclass);
+
     panel_group_unpack (group);
 
     group_list = g_slist_delete_link (group_list, li);
@@ -484,6 +580,7 @@ groups_remove (int index)
     }
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_show_popup */
 void
 groups_show_popup (int index, gboolean show)
 {
@@ -491,93 +588,29 @@ groups_show_popup (int index, gboolean show)
     PanelGroup *group;
 
     li = g_slist_nth (group_list, index);
-    if (!li) 
-        return;
+    if (!li)
+	return;
 
     group = li->data;
 
-    if (show)
-	gtk_widget_show (group->popup->button);
-    else
-	gtk_widget_hide (group->popup->button);
-}
-
-/* settings */
-void
-groups_set_orientation (int orientation)
-{
-    GSList *li;
-    PanelGroup *group;
-
-    for (li = group_list; li; li = li->next)
+    if (show && group->control->with_popup && !group->popup)
     {
-	group = li->data;
-
-	control_set_orientation (group->control, orientation);
-    }
-}
-
-void
-groups_set_layer (int layer)
-{
-    GSList *li;
-    PanelGroup *group;
-
-    for (li = group_list; li; li = li->next)
-    {
-	group = li->data;
-
-	panel_popup_set_layer (group->popup, layer);
-    }
-}
-
-void
-groups_set_size (int size)
-{
-    GSList *li;
-    PanelGroup *group;
-
-    for (li = group_list; li; li = li->next)
-    {
-	group = li->data;
-
-	panel_popup_set_size (group->popup, size);
-	control_set_size (group->control, size);
-    }
-}
-
-void
-groups_set_popup_position (int position)
-{
-    GSList *li;
-    PanelGroup *group;
-
-    for (li = group_list; li; li = li->next)
-    {
-	group = li->data;
-
+	group->popup = create_panel_popup ();
 	panel_group_arrange (group);
-	panel_popup_set_popup_position (group->popup, position);
     }
-}
 
-void
-groups_set_theme (const char *theme)
-{
-    GSList *li;
-    PanelGroup *group;
-
-    for (li = group_list; li; li = li->next)
+    if (group->popup)
     {
-	group = li->data;
-
-	panel_popup_set_theme (group->popup, theme);
-	control_set_theme (group->control, theme);
+	if (show)
+	    gtk_widget_show (group->popup->button);
+	else
+	    gtk_widget_hide (group->popup->button);
     }
 }
 
+G_MODULE_EXPORT /* EXPORT:groups_add_control */
 void
-groups_add_control (int id, const char *filename, int index)
+groups_add_control (Control * control, int index)
 {
     int len;
     PanelGroup *group = NULL;
@@ -591,104 +624,25 @@ groups_add_control (int id, const char *filename, int index)
     panel_group_pack (group, groupbox);
     group_list = g_slist_append (group_list, group);
 
-    group->popup = create_panel_popup ();
-
-    group->control = control_new (index);
-    create_control (group->control, id, filename);
+    group->control = control;
+    control->index = index;
 
     panel_group_arrange (group);
 
     if (index >= 0 && index < len)
 	groups_move (len, index);
 
-    if (!group->control->with_popup)
-	groups_show_popup (index, FALSE);
+    if (group->control->with_popup)
+    {
+	groups_show_popup (index, TRUE);
+    }
 
     settings.num_groups++;
 }
 
-void
-groups_set_num_groups (int n)
+G_MODULE_EXPORT /* EXPORT:groups_get_n_controls */
+int
+groups_get_n_controls (void)
 {
-    int i, len;
-    GSList *li;
-    PanelGroup *group = NULL;
-
-    len = g_slist_length (group_list);
-
-    /* nothing to do */
-    if (n == len)
-	return;
-
-    /* add group(s) */
-    if (n > len)
-    {
-	for (i = len; i < n; i++)
-	{
-	    if (removed_list)
-	    {
-		group = removed_list->data;
-		removed_list =
-		    g_slist_delete_link (removed_list, removed_list);
-
-		panel_group_pack (group, groupbox);
-		group_list = g_slist_append (group_list, group);
-
-		panel_group_arrange (group);
-		settings.num_groups++;
-	    }
-	    else
-	    {
-		groups_add_control (ICON, NULL, i);
-	    }
-
-	    gtk_widget_show (group->base);
-	}
-
-	return;
-    }
-
-    /* removing group(s) */
-    li = g_slist_nth (group_list, n);
-
-    for (i = n; i < len; i++)
-    {
-	GSList *li2;
-
-	group = li->data;
-	panel_group_unpack (group);
-
-	/* this sets the list to the next group */
-	li2 = li;
-	li = li->next;
-
-	group_list = g_slist_remove_link (group_list, li2);
-	removed_list = g_slist_prepend (removed_list, group);
-	settings.num_groups--;
-    }
+    return g_slist_length (group_list);
 }
-
-/* arrow direction */
-
-void
-groups_set_arrow_direction (GtkArrowType type)
-{
-    GSList *li;
-    PanelGroup *group;
-
-    popup_arrow_type = type;
- 
-    for (li = group_list; li; li = li->next)
-    {
-	group = li->data;
- 
-	panel_popup_set_arrow_type (group->popup, type);
-    }
-}
- 
-GtkArrowType
-groups_get_arrow_direction (void)
-{
-    return popup_arrow_type;
-}
-
