@@ -1,4 +1,4 @@
-/*  $Id: item.c,v 1.46.2.2 2005/01/08 09:04:47 jasper Exp $
+/*  $Id: item.c 17514 2005-09-09 18:44:39Z jasper $
  *  
  *  Copyright 2002-2004 Jasper Huijsmans (jasper@xfce.org)
  *
@@ -49,24 +49,6 @@
 static gboolean popup_from_timeout = FALSE;
 static int popup_timeout_id = 0;
 
-/* prevent users from clicking twice because of slow app start */
-static gboolean set_sensitive_cb (GtkWidget *button)
-{
-    if (GTK_IS_WIDGET (button))
-        gtk_widget_set_sensitive (button, TRUE);
-    
-    return FALSE;
-}
-
-static void
-after_exec_insensitive (GtkWidget *button)
-{
-    gtk_widget_set_sensitive (button, FALSE);
-    
-    g_timeout_add (INSENSITIVE_TIMEOUT, 
-                   (GSourceFunc) set_sensitive_cb, button);
-}
-
 /*  Common item callbacks
  *  ---------------------
 */
@@ -106,7 +88,6 @@ item_drop_cb (GtkWidget * widget, GdkDragContext * context, gint x,
 	    g_string_append_c (execute, '"');
 
 	exec_cmd (execute->str, item->in_terminal, item->use_sn);
-        after_exec_insensitive (item->button);
         
 	g_string_free (execute, TRUE);
 
@@ -133,7 +114,6 @@ item_click_cb (GtkButton * b, Item * item)
     hide_current_popup_menu ();
 
     exec_cmd (item->command, item->in_terminal, item->use_sn);
-    after_exec_insensitive (item->button);
 }
 
 static gboolean
@@ -202,7 +182,7 @@ addtomenu_item_drop_cb (GtkWidget * widget,
                                        xfce_desktop_entry_new (buf, keys, 
                                            G_N_ELEMENTS (keys))))
             {
-                char *term;
+                char *term = NULL;
                 
                 xfce_desktop_entry_get_string (dentry, "GenericName", FALSE,
                                                &(mi->caption));
