@@ -1,6 +1,6 @@
-/*  xfce4
+/*  $Id: main.h 4135 2004-09-13 20:31:39Z jasper $
  *
- *  Copyright (C) 2002 Jasper Huijsmans <huysmans@users.sourceforge.net>
+ *  Copyright 2002-2004 Jasper Huijsmans <jasper@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,9 @@
 #ifndef __XFCE_MAIN_H__
 #define __XFCE_MAIN_H__
 
-void quit (gboolean force);
-void restart (void);
+#include <gmodule.h>
+
+G_MODULE_IMPORT void quit (gboolean force);
+G_MODULE_IMPORT void restart (void);
 
 #endif /* __XFCE_MAIN_H__ */
